@@ -25,6 +25,12 @@
 
 PyObject* ToxCoreError;
 
+#define CHECK_TOX(self)                                        \
+  if ((self)->tox == NULL) {                                   \
+    PyErr_SetString(ToxCoreError, "toxcore object killed.");   \
+    return NULL;                                               \
+  }
+
 static void bytes_to_hex_string(uint8_t* digest, int length,
     uint8_t* hex_digest)
 {
@@ -99,10 +105,10 @@ typedef struct {
 static void callback_friend_request(uint8_t* public_key, uint8_t* data,
     uint16_t length, void* self)
 {
-  uint8_t buf[TOX_FRIEND_ADDRESS_SIZE * 2 + 1];
-  memset(buf, 0, TOX_FRIEND_ADDRESS_SIZE * 2 + 1);
+  uint8_t buf[TOX_CLIENT_ID_SIZE * 2 + 1];
+  memset(buf, 0, TOX_CLIENT_ID_SIZE * 2 + 1);
 
-  bytes_to_hex_string(public_key, TOX_FRIEND_ADDRESS_SIZE, buf);
+  bytes_to_hex_string(public_key, TOX_CLIENT_ID_SIZE, buf);
 
   PyObject_CallMethod((PyObject*)self, "on_friend_request", "ss#", buf, data,
       length - 1);
@@ -279,6 +285,10 @@ static int ToxCore_init(ToxCore* self, PyObject* args, PyObject* kwds)
 static int
 ToxCore_dealloc(ToxCore* self)
 {
+  if (self->tox) {
+    tox_kill(self->tox);
+    self->tox = NULL;
+  }
   return 0;
 }
 
@@ -291,6 +301,8 @@ ToxCore_callback_stub(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_getaddress(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t address[TOX_FRIEND_ADDRESS_SIZE];
   uint8_t address_hex[TOX_FRIEND_ADDRESS_SIZE * 2 + 1];
   memset(address_hex, 0, TOX_FRIEND_ADDRESS_SIZE * 2 + 1);
@@ -305,6 +317,8 @@ ToxCore_getaddress(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_add_friend(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t* address = NULL;
   int addr_length = 0;
   uint8_t* data = NULL;
@@ -362,6 +376,8 @@ ToxCore_add_friend(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_add_friend_norequest(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t* address = NULL;
   int addr_length = 0;
 
@@ -372,7 +388,6 @@ ToxCore_add_friend_norequest(ToxCore* self, PyObject* args)
   uint8_t pk[TOX_FRIEND_ADDRESS_SIZE];
   hex_string_to_bytes(address, TOX_FRIEND_ADDRESS_SIZE, pk);
 
-  // return friend_id on success, else -1.
   int res = tox_add_friend_norequest(self->tox, pk);
   if (res == -1) {
     PyErr_SetString(ToxCoreError, "failed to add friend");
@@ -385,6 +400,8 @@ ToxCore_add_friend_norequest(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_get_friend_id(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t* id = NULL;
   int addr_length = 0;
 
@@ -407,6 +424,8 @@ ToxCore_get_friend_id(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_get_client_id(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t pk[TOX_CLIENT_ID_SIZE + 1];
   pk[TOX_CLIENT_ID_SIZE] = 0;
 
@@ -428,6 +447,8 @@ ToxCore_get_client_id(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_delfriend(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendid = 0;
 
   if (!PyArg_ParseTuple(args, "i", &friendid)) {
@@ -445,6 +466,8 @@ ToxCore_delfriend(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_get_friend_connection_status(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendid = 0;
 
   if (!PyArg_ParseTuple(args, "i", &friendid)) {
@@ -453,7 +476,7 @@ ToxCore_get_friend_connection_status(ToxCore* self, PyObject* args)
 
   int ret = tox_get_friend_connection_status(self->tox, friendid);
   if (ret == -1) {
-    PyErr_SetString(ToxCoreError, "failed get connection status");
+    PyErr_SetString(ToxCoreError, "failed to get connection status");
     return NULL;
   }
 
@@ -463,6 +486,8 @@ ToxCore_get_friend_connection_status(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_friend_exists(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendid = 0;
 
   if (!PyArg_ParseTuple(args, "i", &friendid)) {
@@ -477,6 +502,8 @@ ToxCore_friend_exists(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_sendmessage(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendid = 0;
   int length = 0;
   uint8_t* message = NULL;
@@ -497,6 +524,8 @@ ToxCore_sendmessage(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_sendmessage_withid(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendid = 0;
   int length = 0;
   uint32_t id = 0;
@@ -519,6 +548,8 @@ ToxCore_sendmessage_withid(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_send_action(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendid = 0;
   int length = 0;
   uint8_t* action = NULL;
@@ -539,6 +570,8 @@ ToxCore_send_action(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_send_action_withid(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendid = 0;
   int length = 0;
   uint32_t id = 0;
@@ -561,6 +594,8 @@ ToxCore_send_action_withid(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_set_name(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t* name = 0;
   int length = 0;
 
@@ -579,6 +614,8 @@ ToxCore_set_name(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_get_self_name(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t buf[TOX_MAX_NAME_LENGTH];
   memset(buf, 0, TOX_MAX_NAME_LENGTH);
 
@@ -593,6 +630,8 @@ ToxCore_get_self_name(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_get_name(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendid = 0;
   uint8_t buf[TOX_MAX_NAME_LENGTH];
   memset(buf, 0, TOX_MAX_NAME_LENGTH);
@@ -612,6 +651,8 @@ ToxCore_get_name(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_set_status_message(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t* message;
   int length;
   if (!PyArg_ParseTuple(args, "s#", &message, &length)) {
@@ -629,6 +670,8 @@ ToxCore_set_status_message(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_set_user_status(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int status = 0;
 
   if (!PyArg_ParseTuple(args, "i", &status)) {
@@ -647,6 +690,8 @@ ToxCore_set_user_status(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_get_status_message_size(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendid = 0;
   if (!PyArg_ParseTuple(args, "i", &friendid)) {
     return NULL;
@@ -659,6 +704,8 @@ ToxCore_get_status_message_size(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_get_status_message(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t buf[TOX_MAX_STATUSMESSAGE_LENGTH];
   int friendid = 0;
 
@@ -684,6 +731,8 @@ ToxCore_get_status_message(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_get_self_status_message(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t buf[TOX_MAX_STATUSMESSAGE_LENGTH];
   memset(buf, 0, TOX_MAX_STATUSMESSAGE_LENGTH);
 
@@ -703,6 +752,8 @@ ToxCore_get_self_status_message(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_get_user_status(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendid = 0;
   if (!PyArg_ParseTuple(args, "i", &friendid)) {
     return NULL;
@@ -716,6 +767,8 @@ ToxCore_get_user_status(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_get_self_user_status(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int status = tox_get_self_user_status(self->tox);
   return PyLong_FromLong(status);
 }
@@ -723,6 +776,8 @@ ToxCore_get_self_user_status(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_set_send_receipts(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendid = 0;
   int yesno = 0;
 
@@ -738,6 +793,8 @@ ToxCore_set_send_receipts(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_count_friendlist(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint32_t count = tox_count_friendlist(self->tox);
   return PyLong_FromUnsignedLong(count);
 }
@@ -745,6 +802,8 @@ ToxCore_count_friendlist(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_get_friendlist(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   PyObject* plist = NULL;
   uint32_t count = tox_count_friendlist(self->tox);
   int* list = (int*)malloc(count * sizeof(int));
@@ -767,9 +826,11 @@ ToxCore_get_friendlist(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_add_groupchat(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int ret = tox_add_groupchat(self->tox);
   if (ret == -1) {
-    PyErr_SetString(PyExc_TypeError, "failed to add groupchat");
+    PyErr_SetString(ToxCoreError, "failed to add groupchat");
   }
 
   return PyLong_FromLong(ret);
@@ -778,6 +839,8 @@ ToxCore_add_groupchat(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_del_groupchat(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int groupid = 0;
 
   if (!PyArg_ParseTuple(args, "i", &groupid)) {
@@ -785,7 +848,7 @@ ToxCore_del_groupchat(ToxCore* self, PyObject* args)
   }
 
   if (tox_del_groupchat(self->tox, groupid) == -1) {
-    PyErr_SetString(PyExc_TypeError, "failed to del groupchat");
+    PyErr_SetString(ToxCoreError, "failed to del groupchat");
   }
 
   Py_RETURN_NONE;
@@ -794,6 +857,8 @@ ToxCore_del_groupchat(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_group_peername(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t buf[TOX_MAX_NAME_LENGTH];
   memset(buf, 0, TOX_MAX_NAME_LENGTH);
 
@@ -805,7 +870,7 @@ ToxCore_group_peername(ToxCore* self, PyObject* args)
 
   int ret = tox_group_peername(self->tox, groupid, peernumber, buf);
   if (ret == -1) {
-    PyErr_SetString(PyExc_TypeError, "failed to get group peername");
+    PyErr_SetString(ToxCoreError, "failed to get group peername");
   }
 
   return PyUnicodeString_FromString((const char*)buf);
@@ -814,6 +879,8 @@ ToxCore_group_peername(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_invite_friend(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendnumber = 0;
   int groupid = 0;
   if (!PyArg_ParseTuple(args, "ii", &friendnumber, &groupid)) {
@@ -821,7 +888,7 @@ ToxCore_invite_friend(ToxCore* self, PyObject* args)
   }
 
   if (tox_invite_friend(self->tox, friendnumber, groupid) == -1) {
-    PyErr_SetString(PyExc_TypeError, "failed to invite friend");
+    PyErr_SetString(ToxCoreError, "failed to invite friend");
   }
 
   Py_RETURN_NONE;
@@ -830,6 +897,8 @@ ToxCore_invite_friend(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_join_groupchat(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t* public_key = NULL;
   int length = 0;
   int friendnumber = 0;
@@ -843,7 +912,7 @@ ToxCore_join_groupchat(ToxCore* self, PyObject* args)
 
   int ret = tox_join_groupchat(self->tox, friendnumber, pk);
   if (ret == -1) {
-    PyErr_SetString(PyExc_TypeError, "failed to join group chat");
+    PyErr_SetString(ToxCoreError, "failed to join group chat");
   }
 
   return PyLong_FromLong(ret);
@@ -852,6 +921,8 @@ ToxCore_join_groupchat(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_group_message_send(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int groupid = 0;
   uint8_t* message = NULL;
   uint32_t length = 0;
@@ -861,7 +932,7 @@ ToxCore_group_message_send(ToxCore* self, PyObject* args)
   }
 
   if (tox_group_message_send(self->tox, groupid, message, length + 1) == -1) {
-    PyErr_SetString(PyExc_TypeError, "failed to send group message");
+    PyErr_SetString(ToxCoreError, "failed to send group message");
   }
 
   Py_RETURN_NONE;
@@ -870,6 +941,8 @@ ToxCore_group_message_send(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_group_number_peers(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int groupid = 0;
 
   if (!PyArg_ParseTuple(args, "i", &groupid)) {
@@ -884,6 +957,8 @@ ToxCore_group_number_peers(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_group_get_names(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int groupid = 0;
   if (!PyArg_ParseTuple(args, "i", &groupid)) {
     return NULL;
@@ -895,7 +970,7 @@ ToxCore_group_get_names(ToxCore* self, PyObject* args)
 
   int n2 = tox_group_get_names(self->tox, groupid, names, n);
   if (n2 == -1) {
-    PyErr_SetString(PyExc_TypeError, "failed to get group member names");
+    PyErr_SetString(ToxCoreError, "failed to get group member names");
     return NULL;
   }
 
@@ -917,6 +992,8 @@ ToxCore_group_get_names(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_count_chatlist(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint32_t n = tox_count_chatlist(self->tox);
   return PyLong_FromUnsignedLong(n);
 }
@@ -924,6 +1001,8 @@ ToxCore_count_chatlist(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_get_chatlist(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   PyObject* plist = NULL;
   uint32_t count = tox_count_chatlist(self->tox);
   int* list = (int*)malloc(count * sizeof(int));
@@ -946,6 +1025,8 @@ ToxCore_get_chatlist(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_new_filesender(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendnumber = 0;
   int filesize = 0;
   uint8_t* filename = 0;
@@ -960,7 +1041,7 @@ ToxCore_new_filesender(ToxCore* self, PyObject* args)
       filename_length);
 
   if (ret == -1) {
-    PyErr_SetString(PyExc_TypeError, "tox_new_file_sender() failed");
+    PyErr_SetString(ToxCoreError, "tox_new_file_sender() failed");
     return NULL;
   }
 
@@ -970,6 +1051,8 @@ ToxCore_new_filesender(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_file_sendcontrol(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendnumber = 0;
   uint8_t send_receive = 0;
   int filenumber = 0;
@@ -987,7 +1070,7 @@ ToxCore_file_sendcontrol(ToxCore* self, PyObject* args)
       filenumber, message_id, data, data_length);
 
   if (ret == -1) {
-    PyErr_SetString(PyExc_TypeError, "tox_file_send_control() failed");
+    PyErr_SetString(ToxCoreError, "tox_file_send_control() failed");
     return NULL;
   }
 
@@ -997,6 +1080,8 @@ ToxCore_file_sendcontrol(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_file_senddata(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendnumber = 0;
   int filenumber = 0;
   uint8_t* data = NULL;
@@ -1012,7 +1097,7 @@ ToxCore_file_senddata(ToxCore* self, PyObject* args)
       data_length);
 
   if (ret == -1) {
-    PyErr_SetString(PyExc_TypeError, "tox_file_send_data() failed");
+    PyErr_SetString(ToxCoreError, "tox_file_send_data() failed");
     return NULL;
   }
 
@@ -1022,6 +1107,8 @@ ToxCore_file_senddata(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_file_data_size(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendnumber = 0;
   if (!PyArg_ParseTuple(args, "i", &friendnumber)) {
     return NULL;
@@ -1030,7 +1117,7 @@ ToxCore_file_data_size(ToxCore* self, PyObject* args)
   int ret = tox_file_data_size(self->tox, friendnumber);
 
   if (ret == -1) {
-    PyErr_SetString(PyExc_TypeError, "tox_file_data_size() failed");
+    PyErr_SetString(ToxCoreError, "tox_file_data_size() failed");
     return NULL;
   }
 
@@ -1038,14 +1125,16 @@ ToxCore_file_data_size(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_file_dataremaining(ToxCore* self, PyObject* args)
+ToxCore_file_data_remaining(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int friendnumber = 0;
   int filenumber = 0;
-  uint8_t send_receive = 0;
+  int send_receive = 0;
 
 
-  if (!PyArg_ParseTuple(args, "iic", &friendnumber, &filenumber,
+  if (!PyArg_ParseTuple(args, "iii", &friendnumber, &filenumber,
         &send_receive)) {
     return NULL;
   }
@@ -1054,7 +1143,7 @@ ToxCore_file_dataremaining(ToxCore* self, PyObject* args)
       send_receive);
 
   if (!ret) {
-    PyErr_SetString(PyExc_TypeError, "tox_file_data_remaining() failed");
+    PyErr_SetString(ToxCoreError, "tox_file_data_remaining() failed");
     return NULL;
   }
 
@@ -1064,6 +1153,8 @@ ToxCore_file_dataremaining(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_bootstrap_from_address(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   int ipv6enabled = TOX_ENABLE_IPV6_DEFAULT;
   uint16_t port = 0;
   uint8_t* public_key = NULL;
@@ -1092,6 +1183,8 @@ ToxCore_bootstrap_from_address(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_isconnected(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   if (tox_isconnected(self->tox)) {
     Py_RETURN_TRUE;
   } else {
@@ -1102,7 +1195,10 @@ ToxCore_isconnected(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_kill(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   tox_kill(self->tox);
+  self->tox = NULL;
 
   Py_RETURN_NONE;
 }
@@ -1110,6 +1206,8 @@ ToxCore_kill(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_do(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   tox_do(self->tox);
 
   if (PyErr_Occurred()) {
@@ -1121,6 +1219,8 @@ ToxCore_do(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_size(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint32_t size = tox_size(self->tox);
 
   return PyLong_FromUnsignedLong(size);
@@ -1129,6 +1229,8 @@ ToxCore_size(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_save(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   PyObject* res = NULL;
   int size = tox_size(self->tox);
   uint8_t* buf = (uint8_t*)malloc(size);
@@ -1139,7 +1241,7 @@ ToxCore_save(ToxCore* self, PyObject* args)
 
   tox_save(self->tox, buf);
 
-  // The PyString_* functions were split in Python 3.* into 
+  // The PyString_* functions were split in Python 3.* into
   // PyUnicode_ for string data and PyBytes_ for binary data
 #if PY_MAJOR_VERSION < 3
   res = PyString_FromStringAndSize((const char*)buf, size);
@@ -1154,6 +1256,8 @@ ToxCore_save(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_save_to_file(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   char* filename;
   if (!PyArg_ParseTuple(args, "s", &filename)) {
     return NULL;
@@ -1186,11 +1290,13 @@ ToxCore_save_to_file(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_load(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   uint8_t* data = NULL;
   int length = 0;
 
   if (!PyArg_ParseTuple(args, "s#", &data, &length)) {
-    PyErr_SetString(PyExc_TypeError, "no data specified");
+    PyErr_SetString(ToxCoreError, "no data specified");
     return NULL;
   }
 
@@ -1204,6 +1310,8 @@ ToxCore_load(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_load_from_file(ToxCore* self, PyObject* args)
 {
+  CHECK_TOX(self);
+
   char* filename = NULL;
 
   if (!PyArg_ParseTuple(args, "s", &filename)) {
@@ -1212,7 +1320,7 @@ ToxCore_load_from_file(ToxCore* self, PyObject* args)
 
   FILE* fp = fopen(filename, "r");
   if (fp == NULL) {
-    PyErr_SetString(PyExc_TypeError,
+    PyErr_SetString(ToxCoreError,
         "tox_load(): failed to open file for reading");
     return NULL;
   }
@@ -1225,7 +1333,7 @@ ToxCore_load_from_file(ToxCore* self, PyObject* args)
   uint8_t* data = (uint8_t*)malloc(length * sizeof(char));
 
   if (fread(data, length, 1, fp) != 1) {
-    PyErr_SetString(PyExc_TypeError,
+    PyErr_SetString(ToxCoreError,
         "tox_load(): corrupted data file");
     return NULL;
   }
@@ -1474,9 +1582,9 @@ PyMethodDef Tox_methods[] = {
     "    :meth:`.set_user_status`"
   },
   {
-    "set_sends_receipts", (PyCFunction)ToxCore_set_send_receipts,
+    "set_send_receipts", (PyCFunction)ToxCore_set_send_receipts,
     METH_VARARGS,
-    "set_sends_receipts(friend_number, yesno)\n"
+    "set_send_receipts(friend_number, yesno)\n"
     "Sets whether we send read receipts for friendnumber. *yesno* should be "
     "a boolean value."
   },
@@ -1525,7 +1633,7 @@ PyMethodDef Tox_methods[] = {
   {
     "group_number_peers", (PyCFunction)ToxCore_group_number_peers, METH_VARARGS,
     "group_number_peers(group_number)\n"
-    "Return the number of peers in the group chat on success."
+    "Return the number of peers in the group chat."
   },
   {
     "group_get_names", (PyCFunction)ToxCore_group_get_names, METH_VARARGS,
@@ -1545,7 +1653,7 @@ PyMethodDef Tox_methods[] = {
   {
     "new_file_sender", (PyCFunction)ToxCore_new_filesender, METH_VARARGS,
     "new_file_sender(friend_number, file_size, filename)\n"
-    "Send a file send request."
+    "Send a file send request. Returns file number to be sent."
   },
   {
     "file_send_control", (PyCFunction)ToxCore_file_sendcontrol, METH_VARARGS,
@@ -1579,7 +1687,7 @@ PyMethodDef Tox_methods[] = {
     "Returns the recommended/maximum size of the filedata you send with "
     ":meth:`.file_send_data`."},
   {
-    "file_data_remaining", (PyCFunction)ToxCore_file_dataremaining,
+    "file_data_remaining", (PyCFunction)ToxCore_file_data_remaining,
     METH_VARARGS,
     "file_data_remaining(friend_number, file_number, send_receive)\n"
     "Give the number of bytes left to be sent/received. *send_receive* is "
